//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//
//
//
//	delib_Dapi_register_commands.cpp
//	project: delib
//
//
//	(c) DEDITEC GmbH, 2013
//	web: http://www.deditec.de/
//	mail: vertrieb@deditec.de
//
// 
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************

// ------------------------------------
// DEBUG OUTPUT 
//#define DEBUG_OUTPUT		1				// Ausgabe aktivieren (nur fr diese CPP-Datei)
// ------------------------------------

// ------------------------------------
// ------------------------------------
// History
// ------------------------------------
// ------------------------------------
//
// DT	12.11.2013	Start DEDITEC-History

#ifdef USE_ETH_INTERN
// Muss sein, wegen return als long long
extern unsigned long long DapiROETHInternReadLongLong(unsigned long handle, unsigned long adress);
#endif

// ------------------------------------
// ------------------------------------
// Functions
// ------------------------------------
// ------------------------------------

// DapiPing
// ------------------------------------
// DapiWriteByte
// DapiWriteWord
// DapiWriteLong
// DapiWriteLongLong
// ------------------------------------
// DapiReadByte
// DapiReadWord
// DapiReadLong
// DapiReadLongLong
// ------------------------------------
// DapiReadMultipleBytes
// DapiWriteMultipleBytes
// ------------------------------------
// DapiReadFifo

// ------------------------------------
// ------------------------------------
// Includes
// ------------------------------------
// ------------------------------------

#include <stdlib.h>
#include <stdio.h>

#include "proj_config.h"

#ifdef __linux__
	// LINUX ---------------
	// LINUX ---------------
	// LINUX ---------------
	#ifdef USE_FTDI
		#include "../usb-simple/ftd2xx.h"
	#endif

	#include "library/delib/delib.h"
	#include "library/delib/delib_intern.h"
	// LINUX ---------------
	// LINUX ---------------
	// LINUX ---------------
#else
	// WINDOWS ---------------
	// WINDOWS ---------------
	// WINDOWS ---------------
	#include <windows.h>
	#include "conio.h"
	#include "malloc.h"
	
	#include "library/ftdi/ftd2xx.h"
	#include "delib_io_seriellCOM.h"
	#include "delib_io_seriellFTDI.h"
	#include "delib_io_ftdi.h"
	#include "delib_io_tcp.h"
	#include "library/delib/delib.h"
	#include "library/delib/delib_intern.h"
	#include "delib_set_get_error.h"
	#include "delib_open_close.h"

//	#define USE_FTDI		1
//	#define USE_ETH			1
//	#define USE_SER			1
	// WINDOWS ---------------
	// WINDOWS ---------------
	// WINDOWS ---------------
#endif

// ------------------------------------
// ------------------------------------
// DELIB-DEBUG
// ------------------------------------
// ------------------------------------
//
// Function															Windows (DEBUGVIEW)			Linux
// -------------------------										----------------------		-------------------------
// DapiSetError(debug_msg)											immer						immer
// debug_error(debug_msg)											immer						immer
// debug_info(debug_msg)											immer						nur bei DEBUG_OUTPUT
// debug_print(debug_msg)											nur bei DEBUG_OUTPUT		nur bei DEBUG_OUTPUT

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

ULONG DAPI_FUNCTION_PRE DapiPing(ULONG handle, ULONG value)
{
	ULONG ret;
	DAPI_HANDLE *DapiHandle;
	DapiHandle = (DAPI_HANDLE*) handle;

	// DEBUG
	#ifdef DEBUG_OUTPUT
		char debug_msg[200];
		debug_set_function_name("DapiPing");
		sprintf(debug_msg,"DapiPing(handle=0x%lx, value=0x%lx)", handle, value);
		debug_print(debug_msg);
	#endif
	// -----
	
	// check if handle is correct
	if(DapiCheckHandle(handle) != 0) 
	{
		return 0;
	}
	
	switch(DapiHandle->BusType)
	{
		#ifdef USE_FTDI
			// ------------------------------------
			case bus_SeriellFTDI:
				DapiSeriellFTDIWriteByte(handle, 0xffff, value);		// Save value at Adress 0xffff
				ret=DapiSeriellFTDIReadByte(handle, 0xffff);
				break;
			// ------------------------------------
			case bus_FTDI_DUAL:
			case bus_FTDI_SINGLE:
				ret = DapiFTDIPing(handle, value);
				break;
			// ------------------------------------
		#endif

		#ifdef USE_SER
		// ------------------------------------
			case bus_SeriellCOM:
				DapiSeriellCOMWriteByte(handle, 0xffff, value);			// Save value at Adress 0xffff
				ret=DapiSeriellCOMReadByte(handle, 0xffff);
				break;
		#endif

		#ifdef USE_ETH
		// ------------------------------------
			case bus_ETH:
				DapiTCPWriteByte(handle, 0xffff, value);				// Save value at Adress 0xffff
				ret=DapiTCPReadByte(handle, 0xffff);
				break;
		#endif
		
		#ifdef USE_ETH_INTERN
		// ------------------------------------
			case bus_ETH_INTERN:
				DapiROETHInternWriteByte(handle, 0xffff, value);		// Save value at Adress 0xffff
				ret=DapiROETHInternReadByte(handle, 0xffff);
				break;
		#endif
		
		// ------------------------------------
		default:
			DapiSetError(DAPI_ERR_FUNCTION_NOT_DEFINED, 0, NULL, __FILE__, __LINE__);					
			break;
		// ------------------------------------
	}
	
	return ret;
}
		
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

void DAPI_FUNCTION_PRE DapiWriteByte(ULONG handle, ULONG adress, ULONG value)
{
	DAPI_HANDLE *DapiHandle;
	DapiHandle = (DAPI_HANDLE*) handle;

	// DEBUG
	#ifdef DEBUG_OUTPUT
		char debug_msg[200];
		debug_set_function_name("DapiWriteByte");
		sprintf(debug_msg,"DapiWriteByte(handle=0x%lx, address=0x%lx, value=0x%lx)", handle, adress, value);
		debug_print(debug_msg);
	#endif
	// -----
	
	// check if handle is correct
	if(DapiCheckHandle(handle) != 0) 
	{
		return;
	}

	switch(DapiHandle->BusType)
	{
		#ifdef USE_FTDI
			// ------------------------------------
			case bus_SeriellFTDI:
				DapiSeriellFTDIWriteByte(handle, adress, value);
				break;
			// ------------------------------------
			case bus_FTDI_DUAL:
			case bus_FTDI_SINGLE:
				DapiFTDIWriteByte(handle, adress, value);
				break;
		#endif

		#ifdef USE_SER
			// ------------------------------------
			case bus_SeriellCOM:
				DapiSeriellCOMWriteByte(handle, adress, value);
				break;
		#endif

		#ifdef USE_ETH
			// ------------------------------------
			case bus_ETH:
				DapiTCPWriteByte(handle, adress, value);
				break;
		#endif
		
		#ifdef USE_ETH_INTERN
			// ------------------------------------
			case bus_ETH_INTERN:
				DapiROETHInternWriteByte(handle, adress, value);
				break;
		#endif

		// ------------------------------------
		default:
			DapiSetError(DAPI_ERR_FUNCTION_NOT_DEFINED, 0, NULL, __FILE__, __LINE__);					
			break;
		// ------------------------------------
	}
}
		
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

void DAPI_FUNCTION_PRE DapiWriteWord(ULONG handle, ULONG adress, ULONG value)
{
	DAPI_HANDLE *DapiHandle;
	DapiHandle = (DAPI_HANDLE*) handle;
	
	// DEBUG
	#ifdef DEBUG_OUTPUT
		char debug_msg[200];
		debug_set_function_name("DapiWriteWord");
		sprintf(debug_msg,"DapiWriteWord(handle=0x%lx, address=0x%lx, value=0x%lx)", handle, adress, value);
		debug_print(debug_msg);
	#endif
	// -----
	
	// check if handle is correct
	if(DapiCheckHandle(handle) != 0) 
	{
		return;
	}

	switch(DapiHandle->BusType)
	{
		#ifdef USE_FTDI
			// ------------------------------------
			case bus_SeriellFTDI:
				DapiSeriellFTDIWriteWord(handle, adress, value);
				break;
			// ------------------------------------
			case bus_FTDI_DUAL:
			case bus_FTDI_SINGLE:
				DapiFTDIWriteWord(handle, adress, value);
				break;
		#endif

		#ifdef USE_SER
			// ------------------------------------
			case bus_SeriellCOM:
				DapiSeriellCOMWriteWord(handle, adress, value);
				break;
		#endif

		#ifdef USE_ETH
			// ------------------------------------
			case bus_ETH:
				DapiTCPWriteWord(handle, adress, value);
				break;
		#endif
		
		#ifdef USE_ETH_INTERN
			// ------------------------------------
			case bus_ETH_INTERN:
				DapiROETHInternWriteWord(handle, adress, value);
				break;
		#endif

		// ------------------------------------
		default:
			DapiSetError(DAPI_ERR_FUNCTION_NOT_DEFINED, 0, NULL, __FILE__, __LINE__);					
			break;
		// ------------------------------------
	}
}
		
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

void DAPI_FUNCTION_PRE DapiWriteLong(ULONG handle, ULONG adress, ULONG value)
{
	DAPI_HANDLE *DapiHandle;
	DapiHandle = (DAPI_HANDLE*) handle;

	// DEBUG
	#ifdef DEBUG_OUTPUT
		char debug_msg[200];
		debug_set_function_name("DapiWriteLong");
		sprintf(debug_msg,"DapiWriteLong(handle=0x%lx, address=0x%lx, value=0x%lx)", handle, adress, value);
		debug_print(debug_msg);
	#endif
	// -----
	
	// check if handle is correct
	if(DapiCheckHandle(handle) != 0) 
	{
		return;
	}
	
	switch(DapiHandle->BusType)
	{
		#ifdef USE_FTDI
			// ------------------------------------
			case bus_SeriellFTDI:
				DapiSeriellFTDIWriteLong(handle, adress, value);
				break;
			// ------------------------------------
			case bus_FTDI_DUAL:
			case bus_FTDI_SINGLE:
				DapiFTDIWriteLong(handle, adress, value);
				break;
		#endif

		#ifdef USE_SER
			// ------------------------------------
			case bus_SeriellCOM:
				DapiSeriellCOMWriteLong(handle, adress, value);
				break;
		#endif

		#ifdef USE_ETH
			// ------------------------------------
			case bus_ETH:
				DapiTCPWriteLong(handle, adress, value);
				break;
		#endif
		
		#ifdef USE_ETH_INTERN
			// ------------------------------------
			case bus_ETH_INTERN:
				DapiROETHInternWriteLong(handle, adress, value);
				break;
		#endif

		// ------------------------------------
		default:
			DapiSetError(DAPI_ERR_FUNCTION_NOT_DEFINED, 0, NULL, __FILE__, __LINE__);					
			break;
		// ------------------------------------
	}
}
		
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

void DAPI_FUNCTION_PRE DapiWriteLongLong(ULONG handle, ULONG adress, ULONGLONG value)
{
	DAPI_HANDLE *DapiHandle;
	DapiHandle = (DAPI_HANDLE*) handle;

	// DEBUG
	#ifdef DEBUG_OUTPUT
		char debug_msg[200];
		debug_set_function_name("DapiWriteLongLong");
		#ifdef __linux__
			sprintf(debug_msg,"DapiWriteLongLong(handle=0x%lx, address=0x%lx, value=0x%llx)", handle, adress, value);
		#else
			sprintf(debug_msg,"DapiWriteLongLong(handle=0x%lx, address=0x%lx, value=0x%I64x)", handle, adress, value);
		#endif
		debug_print(debug_msg);
	#endif
	// -----
	
	// check if handle is correct
	if(DapiCheckHandle(handle) != 0) 
	{
		return;
	}

	switch(DapiHandle->BusType)
	{
		#ifdef USE_FTDI
			// ------------------------------------
			case bus_SeriellFTDI:
				DapiSeriellFTDIWriteLongLong(handle, adress, value);
				break;
			// ------------------------------------
			case bus_FTDI_DUAL:
			case bus_FTDI_SINGLE:
				DapiFTDIWriteLongLong(handle, adress, value);
				break;
		#endif


		#ifdef USE_SER
			// ------------------------------------
			case bus_SeriellCOM:
				DapiSeriellCOMWriteLongLong(handle, adress, value);
				break;
		#endif


		#ifdef USE_ETH
			// ------------------------------------
			case bus_ETH:
				DapiTCPWriteLongLong(handle, adress, value);
				break;
		#endif
		
		#ifdef USE_ETH_INTERN
			// ------------------------------------
			case bus_ETH_INTERN:
				DapiROETHInternWriteLongLong(handle, adress, value);
				break;
		#endif

		// ------------------------------------
		default:
			DapiSetError(DAPI_ERR_FUNCTION_NOT_DEFINED, 0, NULL, __FILE__, __LINE__);					
			break;
		// ------------------------------------
	}
}
		
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

ULONG DAPI_FUNCTION_PRE DapiReadByte(ULONG handle, ULONG adress)
{
	ULONG ret;
	DAPI_HANDLE *DapiHandle;
	DapiHandle = (DAPI_HANDLE*) handle;

	// DEBUG
	#ifdef DEBUG_OUTPUT
		char debug_msg[200];
		debug_set_function_name("DapiReadByte");
		sprintf(debug_msg,"DapiReadByte(handle=0x%lx, address=0x%lx)", handle, adress);
		debug_print(debug_msg);
	#endif
	// -----
	
	// check if handle is correct
	if(DapiCheckHandle(handle) != 0) 
	{
		return 0;
	}
	
	switch(DapiHandle->BusType)
	{
		#ifdef USE_FTDI
			// ------------------------------------
			case bus_SeriellFTDI:
				ret = DapiSeriellFTDIReadByte(handle, adress);
				break;
			// ------------------------------------
			case bus_FTDI_DUAL:
			case bus_FTDI_SINGLE:
				ret = DapiFTDIReadByte(handle, adress);
				break;
		#endif

		#ifdef USE_SER
			// ------------------------------------
			case bus_SeriellCOM:
				ret = DapiSeriellCOMReadByte(handle, adress);
				break;
		#endif

		#ifdef USE_ETH
			// ------------------------------------
			case bus_ETH:
				ret = DapiTCPReadByte(handle, adress);
				break;
		#endif
		
		#ifdef USE_ETH_INTERN
			// ------------------------------------
			case bus_ETH_INTERN:
				ret = DapiROETHInternReadByte(handle, adress);
				break;
		#endif

		// ------------------------------------
		default:
			DapiSetError(DAPI_ERR_FUNCTION_NOT_DEFINED, 0, NULL, __FILE__, __LINE__);					
			break;
		// ------------------------------------
	}

	// DEBUG
	#ifdef DEBUG_OUTPUT
		debug_set_function_name("DapiReadByte");
		sprintf(debug_msg,"return=0x%lx", ret);
		debug_print(debug_msg);
	#endif
	// -----
	
	return ret;
}
		
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

ULONG DAPI_FUNCTION_PRE DapiReadWord(ULONG handle, ULONG adress)
{
	ULONG ret;
	DAPI_HANDLE *DapiHandle;
	DapiHandle = (DAPI_HANDLE*) handle;

	// DEBUG
	#ifdef DEBUG_OUTPUT
		char debug_msg[200];
		debug_set_function_name("DapiReadWord");
		sprintf(debug_msg,"DapiReadWord(handle=0x%lx, address=0x%lx)", handle, adress);
		debug_print(debug_msg);
	#endif
	// -----
	
	// check if handle is correct
	if(DapiCheckHandle(handle) != 0) 
	{
		return 0;
	}

	switch(DapiHandle->BusType)
	{
		#ifdef USE_FTDI
			// ------------------------------------
			case bus_SeriellFTDI:
				ret = DapiSeriellFTDIReadWord(handle, adress);
				break;
			// ------------------------------------
			case bus_FTDI_DUAL:
			case bus_FTDI_SINGLE:
				ret = DapiFTDIReadWord(handle, adress);
				break;
		#endif

		#ifdef USE_SER
			// ------------------------------------
			case bus_SeriellCOM:
				ret = DapiSeriellCOMReadWord(handle, adress);
				break;
		#endif

		#ifdef USE_ETH
			// ------------------------------------
			case bus_ETH:
				ret = DapiTCPReadWord(handle, adress);
				break;
		#endif
		
		#ifdef USE_ETH_INTERN
			// ------------------------------------
			case bus_ETH_INTERN:
				ret = DapiROETHInternReadWord(handle, adress);
				break;
		#endif
		
		// ------------------------------------
		default:
			DapiSetError(DAPI_ERR_FUNCTION_NOT_DEFINED, 0, NULL, __FILE__, __LINE__);					
			break;
		// ------------------------------------
	}

	// DEBUG
	#ifdef DEBUG_OUTPUT
		debug_set_function_name("DapiReadWord");
		sprintf(debug_msg,"return=0x%lx", ret);
		debug_print(debug_msg);
	#endif
	// -----

	return ret;
}
		
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

ULONG DAPI_FUNCTION_PRE DapiReadLong(ULONG handle, ULONG adress)
{
	ULONG ret;
	DAPI_HANDLE *DapiHandle;
	DapiHandle = (DAPI_HANDLE*) handle;

	// DEBUG
	#ifdef DEBUG_OUTPUT
		char debug_msg[200];
		debug_set_function_name("DapiReadLong");
		sprintf(debug_msg,"DapiReadLong(handle=0x%lx, address=0x%lx)", handle, adress);
		debug_print(debug_msg);
	#endif
	// -----
	
	// check if handle is correct
	if(DapiCheckHandle(handle) != 0) 
	{
		return 0;
	}

	switch(DapiHandle->BusType)
	{
		#ifdef USE_FTDI
		// ------------------------------------
			case bus_SeriellFTDI:
				ret = DapiSeriellFTDIReadLong(handle, adress);
				break;
			// ------------------------------------
			case bus_FTDI_DUAL:
			case bus_FTDI_SINGLE:
				ret = DapiFTDIReadLong(handle, adress);
				break;
		#endif

		#ifdef USE_SER
			// ------------------------------------
			case bus_SeriellCOM:
				ret = DapiSeriellCOMReadLong(handle, adress);
				break;
		#endif

		#ifdef USE_ETH
			// ------------------------------------
			case bus_ETH:
				ret = DapiTCPReadLong(handle, adress);
				break;
		#endif
		
		#ifdef USE_ETH_INTERN
			// ------------------------------------
			case bus_ETH_INTERN:
				ret = DapiROETHInternReadLong(handle, adress);
				break;
		#endif

		// ------------------------------------
		default:
			DapiSetError(DAPI_ERR_FUNCTION_NOT_DEFINED, 0, NULL, __FILE__, __LINE__);					
			break;
		// ------------------------------------
	}

	// DEBUG
	#ifdef DEBUG_OUTPUT
		debug_set_function_name("DapiReadLong");
		sprintf(debug_msg,"return=0x%lx", ret);
		debug_print(debug_msg);
	#endif
	// -----

	return ret;
}
		
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

ULONGLONG DAPI_FUNCTION_PRE DapiReadLongLong(ULONG handle, ULONG adress)
{
	ULONGLONG ret;
	DAPI_HANDLE *DapiHandle;
	DapiHandle = (DAPI_HANDLE*) handle;

	// DEBUG
	#ifdef DEBUG_OUTPUT
		char debug_msg[200];
		debug_set_function_name("DapiReadLongLong");
		sprintf(debug_msg,"DapiReadLongLong(handle=0x%lx, address=0x%lx)", handle, adress);
		debug_print(debug_msg);
	#endif
	// -----
	
	// check if handle is correct
	if(DapiCheckHandle(handle) != 0) 
	{
		return 0;
	}

	switch(DapiHandle->BusType)
	{
		#ifdef USE_FTDI
			// ------------------------------------
			case bus_SeriellFTDI:
				ret = DapiSeriellFTDIReadLongLong(handle, adress);
				break;
			// ------------------------------------
			case bus_FTDI_DUAL:
			case bus_FTDI_SINGLE:
				ret = DapiFTDIReadLongLong(handle, adress);
				break;
		#endif

		#ifdef USE_SER
			// ------------------------------------
			case bus_SeriellCOM:
				ret = DapiSeriellCOMReadLongLong(handle, adress);
				break;
		#endif

		#ifdef USE_ETH
			// ------------------------------------
			case bus_ETH:
				ret = DapiTCPReadLongLong(handle, adress);
				break;
		#endif
		
		#ifdef USE_ETH_INTERN
			// ------------------------------------
			case bus_ETH_INTERN:
				ret = DapiROETHInternReadLongLong(handle, adress);
				break;
		#endif

		// ------------------------------------
		default:
			DapiSetError(DAPI_ERR_FUNCTION_NOT_DEFINED, 0, NULL, __FILE__, __LINE__);					
			break;
		// ------------------------------------
	}

	// DEBUG
	#ifdef DEBUG_OUTPUT
		debug_set_function_name("DapiReadLongLong");
		#ifdef __linux__
			sprintf(debug_msg,"return=0x%llx", ret);
		#else
			sprintf(debug_msg,"return=0x%I64x", ret);
		#endif
		debug_print(debug_msg);
	#endif
	// -----

	return ret;
}
		
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

ULONG DAPI_FUNCTION_PRE DapiReadMultipleBytes(ULONG handle, ULONG adress, ULONG adress_depth, ULONG repeat, UCHAR * buff, ULONG buff_len)
{
	ULONG ret;
	DAPI_HANDLE *DapiHandle;
	DAPI_MODULE_PARAMS *DapiModuleParams;
	char debug_msg[200];
	
	DapiHandle = (DAPI_HANDLE*) handle;
	DapiModuleParams = (DAPI_MODULE_PARAMS *) DapiHandle->addr_dapi_module_params;

	// DEBUG
	#ifdef DEBUG_OUTPUT
		debug_set_function_name("DapiReadMultipleBytes");
		sprintf(debug_msg,"DapiReadMultipleBytes(handle=0x%lx, address=0x%lx, address_depth=0x%lx, repeat=0x%lx, buff_len=0x%lx)", handle, adress, adress_depth, repeat, buff_len);
		debug_print(debug_msg);
	#endif
	// -----
	
	// check if handle is correct
	if(DapiCheckHandle(handle) != 0) 
	{
		return 0;
	}

	switch(DapiHandle->BusType)
	{
		#ifdef USE_FTDI
		// ------------------------------------
			case bus_SeriellFTDI:
				#ifndef __linux__
					if (buff_len > 64)
					{
						sprintf(debug_msg, "DapiSeriellFTDIRead64BytesMax: buffer too large * buff_len = %ld [dec]", buff_len);
						DapiSetError(DAPI_ERR_WITH_TEXT, 0, debug_msg, __FILE__, __LINE__);
						return RETURN_ERROR;
					}
				
					DapiSeriellFTDIRead64BytesMax(handle, adress, buff, adress_depth);
				#endif
				break;
			// ------------------------------------
			case bus_FTDI_DUAL:
			case bus_FTDI_SINGLE:
				ret = DapiFTDIReadMultipleBytes(handle, adress, adress_depth, repeat, buff, buff_len);
				break;
		#endif

		#ifdef USE_SER
			// ------------------------------------
			case bus_SeriellCOM:			
				if (buff_len > 64)
				{
					sprintf(debug_msg, "DapiSeriellCOMRead64BytesMax: buffer too large * buff_len = %ld [dec]", buff_len);
					DapiSetError(DAPI_ERR_WITH_TEXT, 0, debug_msg, __FILE__, __LINE__);
					return RETURN_ERROR;
				}
				
				DapiSeriellCOMRead64BytesMax(handle, adress, buff, adress_depth);
				break;
		#endif
		
		#ifdef USE_ETH
			// ------------------------------------
			case bus_ETH:
				// only ro-eth-modules with software-features support this command!
				if ((DapiModuleParams->sw_feature1 & DAPI_SW_FEATURE_BIT_SUPPORTED_BY_FIRMWARE) == 0)
				{
					sprintf(debug_msg, "DapiReadMultipleBytes -> Module does not support this command");
					DapiSetError(DAPI_ERR_WITH_TEXT, 0, debug_msg, __FILE__, __LINE__);
					return RETURN_ERROR;
				}
			
				ret = DapiTCPReadMultipleBytes(handle, adress, adress_depth, repeat, buff, buff_len);
				break;
		#endif
		
		#ifdef USE_ETH_INTERN
			// ------------------------------------
			case bus_ETH_INTERN:
				ret = DapiROETHInternReadMultipleBytes(handle, adress, adress_depth, repeat, buff, buff_len);
				break;
		#endif
		
		// ------------------------------------
		default:
			DapiSetError(DAPI_ERR_FUNCTION_NOT_DEFINED, 0, NULL, __FILE__, __LINE__);					
			break;
		// ------------------------------------
	}
	
	// DEBUG
	#ifdef DEBUG_OUTPUT
		debug_set_function_name("DapiReadMultipleBytes");
		sprintf(debug_msg,"return=0x%lx", ret);
		debug_print(debug_msg);
	#endif
	// -----
	
	return ret;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

ULONG DAPI_FUNCTION_PRE DapiWriteMultipleBytes(ULONG handle, ULONG adress, ULONG adress_depth, ULONG repeat, UCHAR * buff, ULONG buff_len)
{
	ULONG ret;
	DAPI_HANDLE *DapiHandle;
	DAPI_MODULE_PARAMS *DapiModuleParams;
	char debug_msg[200];
	ULONG len;
	ULONG pos;
	
	DapiHandle = (DAPI_HANDLE*) handle;
	DapiModuleParams = (DAPI_MODULE_PARAMS *) DapiHandle->addr_dapi_module_params;

	// DEBUG
	#ifdef DEBUG_OUTPUT
		debug_set_function_name("DapiWriteMultipleBytes");
		sprintf(debug_msg,"DapiWriteMultipleBytes(handle=0x%lx, address=0x%lx, address_depth=0x%lx, repeat=0x%lx, buff_len=0x%lx)", handle, adress, adress_depth, repeat, buff_len);
		debug_print(debug_msg);
	#endif
	// -----
	
	// check if handle is correct
	if(DapiCheckHandle(handle) != 0) 
	{
		return 0;
	}

	switch(DapiHandle->BusType)
	{
		#ifdef USE_FTDI
			// ------------------------------------
			case bus_SeriellFTDI:
				if (buff_len > 1024)
				{
					sprintf(debug_msg, "DapiSeriellFTDIWrite64BytesMax: buffer too large ( > 1024 Bytes) * buff_len = %ld [dec]", buff_len);
					DapiSetError(DAPI_ERR_WITH_TEXT, 0, debug_msg, __FILE__, __LINE__);
					return RETURN_ERROR;
				}
				
				len = buff_len;
				pos = 0;
				while(len > 0)
				{
					if(len > 32)// Mit 32 klappts besser !
					{
						DapiSeriellFTDIWrite64BytesMax(handle, adress + pos, buff + pos, 32);
						pos += 32;
						len -= 32;
					}
					else
					{
						DapiSeriellFTDIWrite64BytesMax(handle, adress + pos, buff + pos, len);
						pos += len;
						len = 0;
					}
				}
				break;
			// ------------------------------------
			case bus_FTDI_DUAL:
			case bus_FTDI_SINGLE:
				ret = DapiFTDIWriteMultipleBytes(handle, adress, adress_depth, repeat, buff, buff_len);
				break;
		#endif

		#ifdef USE_SER
			// ------------------------------------
			case bus_SeriellCOM:
				if (buff_len > 1024)
				{
					sprintf(debug_msg, "DapiSeriellCOMWrite64BytesMax: buffer too large ( > 1024 Bytes) * buff_len = %ld [dec]", buff_len);
					DapiSetError(DAPI_ERR_WITH_TEXT, 0, debug_msg, __FILE__, __LINE__);
					return RETURN_ERROR;
				}
			
				len = buff_len;
				pos = 0;
				while(len > 0)
				{
					if(len > 64)
					{
						DapiSeriellCOMWrite64BytesMax(handle, adress + pos, buff + pos, 64);
						pos += 64;
						len -= 64;
					}
					else
					{
						DapiSeriellCOMWrite64BytesMax(handle, adress + pos, buff + pos, len);
						pos += len;
						len = 0;
					}
				}
				break;
		#endif

		#ifdef USE_ETH
			// ------------------------------------
			case bus_ETH:
				// only ro-eth-modules with software-features support this command!
				if ((DapiModuleParams->sw_feature1 & DAPI_SW_FEATURE_BIT_SUPPORTED_BY_FIRMWARE) == 0)
				{
					//sprintf(debug_msg, "DapiWriteMultipleBytes -> Module does not support this command");
					//DapiSetError(DAPI_ERR_WITH_TEXT, 0, debug_msg, __FILE__, __LINE__);
					//return RETURN_ERROR;
					ret = DapiTCPWriteMultipleBytesOLD(handle, adress, adress_depth, repeat, buff, buff_len);
				}
				else
				{
					ret = DapiTCPWriteMultipleBytes(handle, adress, adress_depth, repeat, buff, buff_len);
				}
				break;
		#endif
		
		#ifdef USE_ETH_INTERN
			// ------------------------------------
			case bus_ETH_INTERN:
				ret = DapiROETHInternWriteMultipleBytes(handle, adress, adress_depth, repeat, buff, buff_len);
				break;
		#endif

		// ------------------------------------
		default:
			DapiSetError(DAPI_ERR_FUNCTION_NOT_DEFINED, 0, NULL, __FILE__, __LINE__);					
			break;
		// ------------------------------------
	}

	// DEBUG
	#ifdef DEBUG_OUTPUT
		debug_set_function_name("DapiWriteMultipleBytes");
		sprintf(debug_msg,"return=0x%lx", ret);
		debug_print(debug_msg);
	#endif
	// -----
	
	return ret;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

ULONG DAPI_FUNCTION_PRE DapiReadFifo(ULONG handle, ULONG type, UCHAR * buff, ULONG buff_len)
{
	ULONG ret;
	DAPI_HANDLE *DapiHandle;
	DapiHandle = (DAPI_HANDLE*) handle;

	// DEBUG
	#ifdef DEBUG_OUTPUT
		char debug_msg[200];
		debug_set_function_name("DapiReadFifo");
		sprintf(debug_msg,"DapiReadFifo(handle=0x%lx, type=0x%lx, buff_len=0x%lx)", handle, type, buff_len);
		debug_print(debug_msg);
	#endif
	// -----
	
	// check if handle is correct
	if(DapiCheckHandle(handle) != 0) 
	{
		return 0;
	}

	switch(DapiHandle->BusType)
	{
		#ifdef USE_ETH
			// ------------------------------------
			case bus_ETH:
				ret = DapiTCPReadFifo(handle, type, buff, buff_len);
				break;
		#endif

		// ------------------------------------
		default:
			DapiSetError(DAPI_ERR_FUNCTION_NOT_DEFINED, 0, NULL, __FILE__, __LINE__);					
			break;
		// ------------------------------------
	}
	
	// DEBUG
	#ifdef DEBUG_OUTPUT
		debug_set_function_name("DapiReadFifo");
		sprintf(debug_msg,"return=0x%lx", ret);
		debug_print(debug_msg);
	#endif
	// -----
	
	return ret;
}

