//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//
//
//
//	deditec_msgqueue.c
//	project: DEDITEC library
//
//
//	(c) DEDITEC GmbH, 2013
//	web: http://www.deditec.de/
//	mail: vertrieb@deditec.de
//
// 
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************

// ------------------------------------
// ------------------------------------
// History
// ------------------------------------
// ------------------------------------
//
// DT	12.11.2013	Start DEDITEC-History
// CC	27.11.2013	Added new functions dt_msgq_get_no_wait, dt_msgq_create_without_creating_thread and 



// ------------------------------------
// ------------------------------------
// Functions
// ------------------------------------
// ------------------------------------

// dt_msgq_create
// dt_msgq_delete
// dt_msgq_global_exists
// dt_msgq_global_create
// dt_msgq_global_read
// dt_msgq_global_delete
// dt_msgq_create_all_for_server
// dt_msgq_create_all_for_client
// dt_msgq_delete_all
// dt_msgq_get
// dt_msgq_get_no_wait
// dt_msgq_put
// dt_msgq_send_message_to_queue_with_init_and_close
// dt_msgq_server_check
//
// ------------------------------------
// ------------------------------------
// Includes
// ------------------------------------
// ------------------------------------

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <unistd.h>
#include <stdio.h>
#include "deditec_msgqueue.h"
#include "../embedded/dt_var_defines.h"
#include "../vc/deditec_string_utils.h"

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

int dt_msgq_create(int permission)
{
	return msgget(IPC_PRIVATE, permission | IPC_CREAT);
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

void dt_msgq_delete(int id)
{
	msgctl(id, IPC_RMID, 0);
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long dt_msgq_global_exists(char* filename)
{
	return (access(filename, F_OK) == 0) ? RETURN_OK : RETURN_ERROR;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long dt_msgq_global_create(int id, char* filename)
{
	FILE *fh = fopen(filename, "w+");
	if (!fh)
	{
		return RETURN_ERROR;
	}
	if (!fwrite(&id, sizeof(int), 1, fh))
	{
		fclose(fh);
		return RETURN_ERROR;
	}
	fclose(fh);
	chmod(filename, 644);
	return RETURN_OK;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

int dt_msgq_global_read(char* filename)
{
	int id;
	FILE *fh;
	while (dt_msgq_global_exists(filename) != RETURN_OK)
	{
		usleep(500);
	}
	while (!(fh = fopen(filename, "r")))
	{
		usleep(500);
	}
	if (!fread(&id, sizeof(int), 1, fh))
	{
		fclose(fh);
		return -1;
	}
	fclose(fh);
	return id;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

void dt_msgq_global_delete (char* filename)
{
	remove(filename);
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long dt_msgq_create_all_for_server(char* filename, int *serverid)
{

	if (dt_msgq_global_exists(filename) == RETURN_OK)
	{
		*serverid = dt_msgq_global_read(filename);
	}
	else
	{
		*serverid = dt_msgq_create(DT_MSGQ_PERM);

		if (*serverid < 0)
		{
			return RETURN_ERROR;
		}
		
		if (dt_msgq_global_create(*serverid, filename) != RETURN_OK)
		{
			dt_msgq_delete(*serverid);
			return RETURN_ERROR;
		}
	}	
	return RETURN_OK;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long dt_msgq_create_all_for_client(int permission, char* filename, int *clientid, int *serverid)
{
	// ---------------------------------
	// init queue
	if(dt_msgq_global_exists(filename) != RETURN_OK)
	{
		return RETURN_ERROR;	
	}
	
	*serverid = dt_msgq_global_read(filename);
	if (*serverid < 0)
	{		
		return RETURN_ERROR;	
	}

	*clientid = dt_msgq_create(permission);
	if (*clientid < 0)
	{	
		return RETURN_ERROR;
	}

	return RETURN_OK;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

void dt_msgq_delete_all(char* filename, int serverid)
{
	remove(filename);
	msgctl(serverid, IPC_RMID, 0);
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long dt_msgq_get(int id, void *retstr, unsigned long retstr_len, unsigned long *no_of_bytes_received)
{
	long ret;

	//If successful, msgrcv() returns the number of bytes actually placed into mtext. On failure, msgrcv() returns a value of -1, receives no message
	ret = msgrcv(id, retstr, retstr_len, 0, 0);
	if(ret == -1)
	{
		return RETURN_ERROR;
	}

	*no_of_bytes_received = ret;
	return RETURN_OK;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long dt_msgq_get_no_wait(int id, void *retstr, unsigned long retstr_len, unsigned long *no_of_bytes_received)
{
	long ret;

	//If successful, msgrcv() returns the number of bytes actually placed into mtext. On failure, msgrcv() returns a value of -1, receives no message
	ret = msgrcv(id, retstr, retstr_len, 0, IPC_NOWAIT);
	if(ret == -1)
	{
		return RETURN_ERROR;
	}

	*no_of_bytes_received = ret;
	return RETURN_OK;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long dt_msgq_put(int id, void *sendstr, unsigned long sendstr_len)
{
	//If successful, msgsnd() returns a value of zero. On failure, msgsnd() returns a value of -1
	return (msgsnd(id, sendstr, sendstr_len, 0) == -1) ? RETURN_ERROR : RETURN_OK;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long dt_msgq_send_message_to_queue_with_init_and_close(char *dest_DT_MSGQ_DEFINE, dt_msgq_c2s *putdata, dt_msgq_s2c *getdata)
{
	int serverid, clientid;
	unsigned long no_of_bytes_received;

	// ---------------------------------
	// init queue

	if(dt_msgq_create_all_for_client(DT_MSGQ_PERM, dest_DT_MSGQ_DEFINE, &clientid, &serverid) != RETURN_OK)
	{	
		return RETURN_ERROR;
	}
	
	putdata->priority = 1;
	putdata->content.clientid = clientid;
	
	// ---------------------------------
	// message an serverid queue schicken
	
	if (dt_msgq_put(serverid, putdata, ((sizeof(dt_msgq_c2s_s) - C2S_VALUE_MAX + (*putdata).content.vallen))) != RETURN_OK)
	{		
		dt_msgq_delete(clientid);
		return RETURN_ERROR;//RET_COMMUNICATIONISSUE;
	}

	// ---------------------------------
	// antwort
	
	if (dt_msgq_get(clientid, getdata, sizeof(dt_msgq_s2c_s), &no_of_bytes_received) != RETURN_OK)
	{
		dt_msgq_delete(clientid);
		return RETURN_ERROR;//RET_COMMUNICATIONISSUE;
	}
		
	// ---------------------------------
	// queue wieder schlieen
	
	dt_msgq_delete(clientid);
	
	// ---------------------------------
	// antwort auf fehler berprfen + ret setzen

	if ((*getdata).content.status != RETURN_OK)	// nicht ok
	{
		return RETURN_ERROR;
	}
	else
	{
		return RETURN_OK;
	}
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
#ifdef PROG_NAME

void dt_msgq_server_check(int server_id, unsigned long param1)
{
	dt_msgq_c2s getdata;
	dt_msgq_s2c putdata;
	unsigned long no_of_bytes_received;
	unsigned long ret;
	
	char msg[10000];
	unsigned char return_value[9000];
	unsigned long return_value_length;
	unsigned char cmd_copy[9000];
	unsigned char single_cmd[1000];
	unsigned char single_val[1000];
	unsigned long delimiter_found;
	unsigned long single_val_len;
	
	// --------------------------------
	// Queue has data ?
	
	switch(param1)
	{
		// ----------------------------
		case DT_MSGQ_SERVER_CHECK_WAIT:
			ret = dt_msgq_get        (server_id, &getdata, sizeof(dt_msgq_c2s_s), &no_of_bytes_received);
			break;
		// ----------------------------
		case DT_MSGQ_SERVER_CHECK_NO_WAIT:
			ret = dt_msgq_get_no_wait(server_id, &getdata, sizeof(dt_msgq_c2s_s), &no_of_bytes_received);
			break;
		// ----------------------------
		default:
			ret = RETURN_ERROR;
			break;
		// ----------------------------
	}
	
	if(ret == RETURN_OK)
	{
		// ----------------------------
		putdata.content.status = RETURN_ERROR;
		putdata.content.vallen = 0;
		putdata.priority = 1;

		switch(getdata.content.rw_type)
		{
			// ----------------------------------------
			// ----------------------------------------
			// ----------------------------------------
			case 'R':
				// berprfe ob cmdlnge  = 0 ist -> dann fehler
				if (getdata.content.cmdlen == 0)
				{
					log_write("msgq_server_check (rw_type=\"R\") -> cmd_length = 0", 1);
					putdata.content.status = RETURN_ERROR;
					break;
				}

				// cmd = ? -> abgefragter wert ist Server-Name
				if(getdata.content.cmdlen == 1)
				{
					if((strcmp_n(getdata.content.cmd, getdata.content.cmdlen, "?", sizeof("?"))) == RETURN_OK)
					{
						// Server-Name;Process-ID;PROG_COUNTER
						sprintf(putdata.content.value, "%s,%d,%ld", PROG_NAME, getpid(), PROG_COUNTER);	
						putdata.content.vallen = strlen_n(putdata.content.value, sizeof(putdata.content.value));
						putdata.content.status = RETURN_OK;
						break;
					}
				}
				
				#ifdef MSGQ_SERVER_CHECK_WITH_COOKIE
					putdata.content.status = request_read_with_cookie(getdata.content.cmd, getdata.content.cmdlen, putdata.content.value, sizeof(putdata.content.value), &putdata.content.vallen, getdata.content.cookie, sizeof(getdata.content.cookie));
				#else
					putdata.content.status = request_read            (getdata.content.cmd, getdata.content.cmdlen, putdata.content.value, sizeof(putdata.content.value), &putdata.content.vallen);
				#endif

				break;

			// ----------------------------------------
			// ----------------------------------------
			// ----------------------------------------
			case 'W':
		
				// berprfe ob (cmdlnge=0 UND vallen=0) ist -> dann fehler
				if ((getdata.content.cmdlen == 0) && (getdata.content.vallen == 0))
				{
					log_write("msgq_server_check (rw_type=\"W\") -> cmd_length=0 AND vallen=0", 1);
					putdata.content.status = RETURN_ERROR;
					break;
				}

				#ifdef MSGQ_SERVER_CHECK_WITH_COOKIE
					putdata.content.status = request_write_with_cookie(getdata.content.cmd, getdata.content.cmdlen, getdata.content.value, getdata.content.vallen, getdata.content.cookie, sizeof(getdata.content.cookie));
				#else
					putdata.content.status = request_write            (getdata.content.cmd, getdata.content.cmdlen, getdata.content.value, getdata.content.vallen);
				#endif

				putdata.content.vallen = 0;

				break;

		// ----------------------------------------
		// ----------------------------------------
		// ----------------------------------------

			case 'M':
				// Multiple read !!!!!!
					
				// ----------------
				// Jetzt werden Parameter gelesen: einer oder mehrere durch Semikolon getrennt

				// berprfe ob val-lnge  = 0 ist -> dann fehler
				if (getdata.content.vallen == 0)
				{
					log_write("msgq_server_check ERROR (CMD=M) -> vallen = 0", 1);
					putdata.content.status = RETURN_ERROR;
					break;
				}

				return_value[0] = 0;	// String terminieren
				// CMD kopieren **** HIER STEHT DAS CMD im value der queue
				strcpy_n(cmd_copy, sizeof(cmd_copy), getdata.content.value, getdata.content.vallen);
				do
				{
					// Aus dem gesamten cmd_copy den Teil bis zum ; in single_cmd kopieren - cmd_copy wird dabei gekrzt !
					delimiter_found = cuttext_n(cmd_copy, strlen_n(cmd_copy, sizeof(cmd_copy)), single_cmd, sizeof(single_cmd), ";", 1);
					// Hier muss noch gendert werden, dass der letzte Parameter auch kopiert wird .... workaround siehe unten
					
					if(delimiter_found == FALSE)
					{
						strcpy_n(single_cmd, sizeof(single_cmd), cmd_copy, strlen_n(cmd_copy, sizeof(cmd_copy)));
					}

					// Auszufhrendes Command steht in:		single_cmd, strlen(single_cmd)
					// Rckgabewert wird geschrieben nach:	single_val, sizeof(single_val), &single_val_len
					
					#ifdef MSGQ_SERVER_CHECK_WITH_COOKIE
						putdata.content.status = request_read_with_cookie(single_cmd, strlen_n(single_cmd, sizeof(single_cmd)), single_val, sizeof(single_val), &single_val_len, getdata.content.cookie, sizeof(getdata.content.cookie));
					#else
						putdata.content.status = request_read            (single_cmd, strlen_n(single_cmd, sizeof(single_cmd)), single_val, sizeof(single_val), &single_val_len);
					#endif
					
					
					//ret =  dt_param_file_read(FILENAME_PARAMETER_IO, strlen(FILENAME_PARAMETER_IO), single_cmd, strlen(single_cmd), single_val, sizeof(single_val), &single_val_len);

					if(ret == RETURN_OK)
					{
						sprintf(msg, "Multi-Param-Read SUCCESS: Param: \"%-40s\" (len=%d) -->> \"%s\" (len=%d)", single_cmd, strlen_n(single_cmd, sizeof(single_cmd)), single_val, single_val_len);
						debug_print(msg);
					}
					else
					{
						sprintf(msg, "\"M\"-Multi-Read #ERROR#: Param: \"%-40s\" (len=%d) -->> **** Parameter could not be found *****", single_cmd, strlen_n(single_cmd, sizeof(single_cmd)));
						log_write(msg, 1);
					} 

					// Wert anhngen (single_val)
					strcat_n(return_value, sizeof(return_value), single_val, strlen_n(single_val, sizeof(single_val)));
					
					if(delimiter_found != FALSE)
					{
						//Trennzeichen anhngen
						strcat_n(return_value, sizeof(return_value), "#@~#", sizeof("#@~#"));
					}
				} while(delimiter_found != FALSE);

				strcpy_n(putdata.content.value, sizeof(putdata.content.value), return_value, strlen_n(return_value, sizeof(return_value)));
				putdata.content.vallen = strlen_n(return_value, sizeof(return_value));
				putdata.content.status = RETURN_OK;
			
				sprintf(msg, "multi-read:retvalue = \"%s\" (len=%d)", putdata.content.value, putdata.content.vallen);
				debug_print(msg);
				
				break;

			// ----------------------------------------
			// ----------------------------------------
			// ----------------------------------------
			default:

				#ifdef MSGQ_SERVER_CHECK_WITH_COOKIE
					putdata.content.status = request_custom_with_cookie(getdata.content.rw_type, getdata.content.cmd, getdata.content.cmdlen, getdata.content.value, getdata.content.vallen, putdata.content.value, sizeof(putdata.content.value), &putdata.content.vallen, getdata.content.cookie, sizeof(getdata.content.cookie));
				#else
					putdata.content.status = request_custom(            getdata.content.rw_type, getdata.content.cmd, getdata.content.cmdlen, getdata.content.value, getdata.content.vallen, putdata.content.value, sizeof(putdata.content.value), &putdata.content.vallen);
				#endif

				break;
				
			// ----------------------------------------
			// ----------------------------------------
			// ----------------------------------------
		}

		// send an answer
		dt_msgq_put(getdata.content.clientid, &putdata, (sizeof(dt_msgq_s2c_s) - S2C_VALUE_MAX + putdata.content.vallen));
	}
}

#endif
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
