//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//
//
//
//	deditec_timer.cpp
//	project: delib
//
//
//  (c) DEDITEC GmbH, 2013
//  web: http://www.deditec.de/
//  mail: vertrieb@deditec.de
//
// 
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************

// ------------------------------------
// ------------------------------------
// History
// ------------------------------------
// ------------------------------------
//
// DT	13.11.2013	Start DEDITEC-History
// CC	26.11.2013	Added 3th timer



// ------------------------------------
// ------------------------------------
// Functions
// ------------------------------------
// ------------------------------------

// dt_timerus_start
// dt2_timerus_start
// dt3_timerus_start
// dt_timerus_diff
// dt2_timerus_diff
// dt3_timerus_diff
// dt_wait_us
// dt2_wait_us
// dt3_wait_us

// ------------------------------------
// ------------------------------------
// Includes
// ------------------------------------
// ------------------------------------

#ifdef __linux__
	// LINUX ---------------
	// LINUX ---------------
	// LINUX ---------------

	#define UCHAR unsigned char
	#define ULONG unsigned long
	#define ULONGLONG unsigned long long

	#include "../embedded/dt_var_defines.h"
	
	#include <errno.h>
	#include <sys/time.h>
	#include <time.h>
#else
	// WINDOWS ---------------
	// WINDOWS ---------------
	// WINDOWS ---------------
	#include "windows.h"
#endif

// ------------------------------------
// ------------------------------------
// Defines
// ------------------------------------
// ------------------------------------

bool dt_timerus_bCounter = FALSE;
bool dt2_timerus_bCounter = FALSE;
bool dt3_timerus_bCounter = FALSE;

#ifdef __linux__
	// LINUX ---------------
	// LINUX ---------------
	// LINUX ---------------
	struct timeval dt_timerus_CounterStart;
	struct timeval dt2_timerus_CounterStart;
	struct timeval dt3_timerus_CounterStart;
#else
	// WINDOWS ---------------
	// WINDOWS ---------------
	// WINDOWS ---------------
	LARGE_INTEGER dt_timerus_Frequency;
	LARGE_INTEGER dt_timerus_CounterStart;
	LARGE_INTEGER dt2_timerus_Frequency;
	LARGE_INTEGER dt2_timerus_CounterStart;
	LARGE_INTEGER dt3_timerus_Frequency;
	LARGE_INTEGER dt3_timerus_CounterStart;
#endif

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

bool dt_timerus_start()
{
	#ifndef __unix__
		dt_timerus_bCounter = QueryPerformanceFrequency(&dt_timerus_Frequency);

		if ( !dt_timerus_bCounter ) return FALSE;

		QueryPerformanceCounter(&dt_timerus_CounterStart);

		return TRUE;
	#else
		if (gettimeofday(&dt_timerus_CounterStart, NULL)) 
			return FALSE;
		else
			return TRUE;
	#endif
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

bool dt2_timerus_start()
{
	#ifndef __unix__
		dt2_timerus_bCounter = QueryPerformanceFrequency(&dt2_timerus_Frequency);

		if ( !dt2_timerus_bCounter ) return FALSE;

		QueryPerformanceCounter(&dt2_timerus_CounterStart);

		return TRUE;
	#else
		if (gettimeofday(&dt2_timerus_CounterStart, NULL)) 
			return FALSE;
		else
			return TRUE;
	#endif
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

bool dt3_timerus_start()
{
	#ifndef __unix__
		dt3_timerus_bCounter = QueryPerformanceFrequency(&dt3_timerus_Frequency);

		if ( !dt_timerus_bCounter ) return FALSE;

		QueryPerformanceCounter(&dt3_timerus_CounterStart);

		return TRUE;
	#else
		if (gettimeofday(&dt3_timerus_CounterStart, NULL)) 
			return FALSE;
		else
			return TRUE;
	#endif
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

ULONG dt_timerus_diff()
{
	#ifndef __unix__
		LARGE_INTEGER CounterCurrent;
		LONGLONG CounterDiff;
		ULONG tdiff_us;

		QueryPerformanceCounter(&CounterCurrent);

		CounterDiff = (LONGLONG) ((LONGLONG)CounterCurrent.QuadPart - (LONGLONG) dt_timerus_CounterStart.QuadPart);

		tdiff_us = (ULONG) (((double) CounterDiff) / ((double) dt_timerus_Frequency.QuadPart) * ((double) 1E6));
		return tdiff_us;
	#else
		struct timeval now;
		ULONG tdiff_us;

		if (gettimeofday(&now, NULL)) 
		{
			tdiff_us = 0;
		} 
		else 
		{
			tdiff_us = (now.tv_sec - dt_timerus_CounterStart.tv_sec) * (1000 * 1000) + (now.tv_usec - dt_timerus_CounterStart.tv_usec);
		}

		return tdiff_us;
	#endif
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

ULONG dt2_timerus_diff()
{
	#ifndef __unix__
		LARGE_INTEGER CounterCurrent;
		LONGLONG CounterDiff;
		ULONG tdiff_us;

		QueryPerformanceCounter(&CounterCurrent);

		CounterDiff = (LONGLONG) ((LONGLONG)CounterCurrent.QuadPart - (LONGLONG) dt2_timerus_CounterStart.QuadPart);

		tdiff_us = (ULONG) (((double) CounterDiff) / ((double) dt2_timerus_Frequency.QuadPart) * ((double) 1E6));
		
		return tdiff_us;
	#else
		struct timeval now;
		ULONG tdiff_us;

		if (gettimeofday(&now, NULL)) 
		{
			tdiff_us = 0;
		}
		else 
		{
			tdiff_us = (now.tv_sec - dt2_timerus_CounterStart.tv_sec) * (1000 * 1000) + (now.tv_usec - dt2_timerus_CounterStart.tv_usec);
		}

		return tdiff_us;
	#endif
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

ULONG dt3_timerus_diff()
{
	#ifndef __unix__
		LARGE_INTEGER CounterCurrent;
		LONGLONG CounterDiff;
		ULONG tdiff_us;

		QueryPerformanceCounter(&CounterCurrent);

		CounterDiff = (LONGLONG) ((LONGLONG)CounterCurrent.QuadPart - (LONGLONG) dt3_timerus_CounterStart.QuadPart);

		tdiff_us = (ULONG) (((double) CounterDiff) / ((double) dt3_timerus_Frequency.QuadPart) * ((double) 1E6));
		
		return tdiff_us;
	#else
		struct timeval now;
		ULONG tdiff_us;

		if (gettimeofday(&now, NULL)) 
		{
			tdiff_us = 0;
		}
		else 
		{
			tdiff_us = (now.tv_sec - dt3_timerus_CounterStart.tv_sec) * (1000 * 1000) + (now.tv_usec - dt3_timerus_CounterStart.tv_usec);
		}

		return tdiff_us;
	#endif
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

void dt_wait_us(ULONG wert)
{
	#ifndef __unix__
		if(dt_timerus_start()==FALSE) return;
		while(dt_timerus_diff()<wert);
	#else
		struct timespec req, rem;
		int result;

		req.tv_sec = wert/(1000 * 1000);
		req.tv_nsec = (wert - req.tv_sec) * 1000;

		do 
		{
			result = nanosleep(&req, &rem);
			
			req.tv_sec = rem.tv_sec;
			req.tv_nsec = rem.tv_nsec;
		} while ((result == -1) && (errno == EINTR));
	#endif
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

void dt2_wait_us(ULONG wert)
{
	#ifndef __unix__
		if(dt2_timerus_start()==FALSE) return;
		while(dt2_timerus_diff()<wert);
	#else
		struct timespec req, rem;
		int result;

		req.tv_sec = wert/(1000 * 1000);
		req.tv_nsec = (wert - req.tv_sec) * 1000;

		do 
		{
			result = nanosleep(&req, &rem);
			
			req.tv_sec = rem.tv_sec;
			req.tv_nsec = rem.tv_nsec;
		} while ((result == -1) && (errno == EINTR));
	#endif
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

void dt3_wait_us(ULONG wert)
{
	#ifndef __unix__
		if(dt3_timerus_start()==FALSE) return;
		while(dt3_timerus_diff()<wert);
	#else
		struct timespec req, rem;
		int result;

		req.tv_sec = wert/(1000 * 1000);
		req.tv_nsec = (wert - req.tv_sec) * 1000;

		do 
		{
			result = nanosleep(&req, &rem);
			
			req.tv_sec = rem.tv_sec;
			req.tv_nsec = rem.tv_nsec;
		} while ((result == -1) && (errno == EINTR));
	#endif
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
