//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//
//
//
//	deditec_parameter_io.c
//	project: DEDITEC library
//
//
//	(c) DEDITEC GmbH, 2013
//	web: http://www.deditec.de/
//	mail: vertrieb@deditec.de
//
// 
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************

#include <stdio.h>

#include "deditec_parameter_io.h"
#include "deditec_msgqueue.h"
#include "deditec_msgqueue_struct.h"
#include "../embedded/dt_embedded_parameter.h"
#include "../embedded/dt_var_defines.h"

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

extern unsigned long parameter_delete(unsigned char *pattern, unsigned long pattern_length)
{
	int serverid, clientid;
	dt_msgq_c2s putdata;
	dt_msgq_s2c getdata;


	if (pattern_length >= sizeof(putdata.content.cmd))
	{
		return RETURN_ERROR;
	}

	// ------------
	// rw_type
	putdata.content.rw_type = 'D';
	// cmd
	putdata.content.cmdlen = pattern_length;
	strcpy_n(putdata.content.cmd, sizeof(putdata.content.cmd), pattern, pattern_length);
	// value
	putdata.content.vallen = 0;
	// ------------

	return dt_msgq_send_message_to_queue_with_init_and_close(DT_MSGQ_PARAM, &putdata, &getdata);
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long parameter_get_text(unsigned char *pattern, unsigned long pattern_length, unsigned char *found_value_buff, unsigned long found_value_buff_size)
{
	int serverid, clientid;
	dt_msgq_c2s putdata;
	dt_msgq_s2c getdata;
	
	*found_value_buff = 0;  // Mit 0 terminieren

	if (pattern_length >= sizeof(putdata.content.cmd))
	{
		return RETURN_ERROR;
	}

	// ------------
	// rw_type
	putdata.content.rw_type = 'R';
	// cmd
	putdata.content.cmdlen = pattern_length;
	strcpy_n(putdata.content.cmd, sizeof(putdata.content.cmd), pattern, pattern_length);
	// value
	putdata.content.vallen = 0;
	// ------------
	
	if(dt_msgq_send_message_to_queue_with_init_and_close(DT_MSGQ_PARAM, &putdata, &getdata) != RETURN_OK)
	{
		return RETURN_ERROR;
	}
	else
	{
		strcpy_n(found_value_buff, found_value_buff_size, getdata.content.value, getdata.content.vallen);
		return RETURN_OK;
	}
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long parameter_get_long(unsigned char *pattern, unsigned long pattern_length, long *found_value)
{
	unsigned char buff[64];
	unsigned long pos = 0;
	long retval = 0;
	unsigned long negative = 0;
	unsigned long returned_value;

	returned_value = parameter_get_text(pattern, pattern_length, buff, sizeof(buff));

	if (RETURN_OK != returned_value)
	{
		return returned_value;
	}

	if (buff[0] == '-')
	{
		negative = 1;
		pos++;
	}
	else if (buff[0] == '+')
	{
		pos++;
	}

	for (; pos < (19 - negative); pos++)
	{
		if (buff[pos] < '0' || buff[pos] > '9')
		{
			if (buff[pos] == 0)
			{
				break;
			}
			else
			{
				return RETURN_ERROR;
			}
		}

		retval = retval * 10 + buff[pos] - '0';
	}

	if (negative)
	{
		*found_value = retval * -1;
		return RETURN_OK;
	}

	*found_value = retval;
	return RETURN_OK;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long parameter_get_float(unsigned char *pattern, unsigned long pattern_length, float *return_value)
{
	unsigned char buff[64];
	unsigned long returned_value;

	returned_value = parameter_get_text (pattern, pattern_length, buff, sizeof(buff));

	if (RETURN_OK != returned_value)
	{
		return returned_value;
	}

	*return_value = (float)atof(buff);

	return RETURN_OK;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long parameter_get_text_or_write_default(unsigned char *pattern, unsigned long pattern_length, unsigned char *found_value_buff, unsigned long found_value_buff_size, unsigned char *default_value_buff, unsigned long default_value_buff_size)
{
	unsigned long ret;

	if((ret = parameter_get_text(pattern, pattern_length, found_value_buff, found_value_buff_size)) != RETURN_OK)
	{
		// Not found -> Set default value now
		ret = parameter_set_text(pattern, pattern_length, default_value_buff, default_value_buff_size);
	}
	
	return ret;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long parameter_get_long_or_write_default(unsigned char *pattern, unsigned long pattern_length, unsigned long *found_value, unsigned long default_value)
{
	unsigned long ret;

	if((ret = parameter_get_long(pattern, pattern_length, found_value)) != RETURN_OK)
	{
		// Not found -> Set default value now
		ret = parameter_set_long(pattern, pattern_length, default_value);
	}
	
	return ret;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long parameter_get_float_or_write_default(unsigned char *pattern, unsigned long pattern_length, float *found_value, float default_value)
{
	unsigned long ret;

	if((ret = parameter_get_float(pattern, pattern_length, found_value)) != RETURN_OK)
	{
		// Not found -> Set default value now
		ret = parameter_set_float(pattern, pattern_length, default_value);
	}
	
	return ret;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long parameter_set_text(unsigned char *pattern, unsigned long pattern_length, unsigned char *insert_value, unsigned long insert_value_length)
{
	int serverid, clientid;
	dt_msgq_c2s putdata;
	dt_msgq_s2c getdata;

	// ------------
	// rw_type
	putdata.content.rw_type = 'W';
	// cmd
	putdata.content.cmdlen = pattern_length;// > sizeof(putdata.content.cmdlen) ? sizeof(putdata.content.cmdlen) : pattern_length;
	strcpy_n(putdata.content.cmd, C2S_VALUE_MAX, pattern, pattern_length);
	
	// Die Terminierung gehrt nicht zur Lnge
	if((putdata.content.cmdlen > 0) && (putdata.content.cmd[putdata.content.cmdlen-1] == 0))
	{
		--putdata.content.cmdlen;
	}
	// value
	putdata.content.vallen = insert_value_length;// > C2S_VALUE_MAX ? C2S_VALUE_MAX : insert_value_length;
	strcpy_n(putdata.content.value, C2S_VALUE_MAX, insert_value, insert_value_length);

	// Die Terminierung gehrt nicht zur Lnge
	if((putdata.content.vallen > 0) && (putdata.content.value[putdata.content.vallen-1] == 0))
	{
		--putdata.content.vallen;
	}
	// ------------

	return dt_msgq_send_message_to_queue_with_init_and_close(DT_MSGQ_PARAM, &putdata, &getdata);
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long parameter_set_long(unsigned char *pattern, unsigned long pattern_length, long insert_value)
{
	char insert_value_c[64];

	sprintf(insert_value_c, "%ld", insert_value);

	return parameter_set_text(pattern, pattern_length, (unsigned char*)insert_value_c, strlen_n(insert_value_c, sizeof(insert_value_c)));
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long parameter_set_float(unsigned char *pattern, unsigned long pattern_length, float insert_value)
{
	char insert_value_c[64];

	sprintf(insert_value_c, "%f", insert_value);

	return parameter_set_text(pattern, pattern_length, (unsigned char*)insert_value_c, strlen_n(insert_value_c, sizeof(insert_value_c)));
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
