//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//
//
//  product: RO-ETH
//  configuration: custom_sample for RO-ETH-Modules
//  programming language: vc
//
//
//  (c) DEDITEC GmbH, 2014
//  web: http://www.deditec.de/
//  mail: vertrieb@deditec.de
//
//
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//
// ------------------------------------
// ------------------------------------
// Includes
// ------------------------------------
// ------------------------------------
//
#include <stdio.h>
#include <stdlib.h>  // usefull for exit(), atoi()
#include <string.h>  // bzero, strlen, strcpy, bcopy
#include <signal.h>
//
#include "proj_config.h"
//
// ------------------------------------
// ------------------------------------
// Global Vars
// ------------------------------------
// ------------------------------------
//
bool signal_handler_flag_shutdown;

//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************

void signal_handler(int sig)
{
	if (sig != SIGTERM)
	{
		return;
	}
	else
	{
		log_write("Signal-handler -> SIGTERM", 1);

		signal_handler_flag_shutdown = TRUE;
	}
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

bool IsError()
{
	unsigned char msg_error[500];
	char msg[500];

	if (DapiGetLastError() != DAPI_ERR_NONE)
	{
		DapiGetLastErrorText((unsigned char*) msg_error, sizeof(msg_error));
		sprintf(msg, "Error Code = %x * Message = %s\n", 0, msg_error);
		log_write(msg, 1);
		
		DapiClearLastError();

		return TRUE;
	}

	return FALSE;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

int main(int argc, char *argv[])
{
	char test_text_variable[20];
	char msg[200];
	
	unsigned long handle;
	unsigned long test_long_variable;
	unsigned long data;
	
	float ad_da_data;
	
	// --------------------------------
	// LOG Init
	log_init("CUST-FW", 1);							// Init Log with pre-text and enable global output
	
	sprintf(msg, "Starting custom1 firmware");		// Writing any String to buffer "msg"
	log_write(msg, 1);								// Writing "msg" buffer to LOG -> see "source/library/linux/deditec_log.h" for further details
	
	// --------------------------------
	// Set Revision No.
	if(versionNr_0 == '0')
	{
		sprintf(msg, "%c.%c%c", versionNr_1, versionNr_2, versionNr_3);		// see file "proj_config.h"
	}
	else
	{
		sprintf(msg, "%c%c.%c%c", versionNr_0, versionNr_1, versionNr_2, versionNr_3);
	}
	
	// --------------------------------
	// Writing variables
	parameter_set_text(CUSTOM1_PARAM_FIRMWARE_VERSION, strlen(CUSTOM1_PARAM_FIRMWARE_VERSION), msg, strlen(msg));							// Writing String
	parameter_set_text(CUSTOM1_PARAM_PRODUCT_NAME, strlen(CUSTOM1_PARAM_PRODUCT_NAME), "MyCustomProject", strlen("MyCustomProject"));		// Writing String
	
	// --------------------------------
	// Reading variables
	parameter_get_text(CUSTOM1_TEST_PARAM_TEXT, strlen(CUSTOM1_TEST_PARAM_TEXT), test_text_variable, sizeof(test_text_variable)); 		// Reading String
	parameter_get_long(CUSTOM1_TEST_PARAM_LONG, strlen(CUSTOM1_TEST_PARAM_LONG), &test_long_variable);									// Reading (long) numeric value
	
	// --------------------------------
	// Print read variables to LOG
	sprintf(msg, "Test Variable 1 = %s\n", test_text_variable);
	log_write(msg, 1);
	
	sprintf(msg, "Test Variable 2 = %ld\n", test_long_variable);
	log_write(msg, 1);
	
	// --------------------------------
	// Init signal_handler
	signal(SIGTERM, signal_handler);
	signal_handler_flag_shutdown = FALSE;
	
	// --------------------------------
	// Open RO-ETH module
	do 
	{
		handle = DapiOpenModule(RO_ETH_INTERN, 0);
		IsError();
		if(handle == 0)
		{
			sprintf(msg, "Error DapiOpenModule -> Handle=0\n");
			log_write(msg, 1);
		
			usleep(1000*1000*1);	// 1 sec retry
		}
	}
	while(handle == 0);

	sprintf(msg, "RO-ETH Module is open\n");
	log_write(msg, 1);
	
	// --------------------------------
	// Selftest - Register Write/Read with all available register-functions
	if (test_readbackAllModes(handle, FALSE) != RETURN_OK)
	{
		test_readbackAllModes(handle, TRUE);
	}
	else
	{
		sprintf(msg, "Register WR Test - OK!\n");
		log_write(msg, 1);
	}
	
	// --------------------------------
	// Init A/D + D/A 
	DapiDASetMode(handle, 0, DAPI_ADDA_MODE_BIPOL_10V);	// Setting D/A CH0 to range 0V..10V
	IsError();
	DapiDASetMode(handle, 1, DAPI_ADDA_MODE_BIPOL_10V);	// Setting D/A CH1 to range 0V..10V
	IsError();
	DapiDASetMode(handle, 2, DAPI_ADDA_MODE_BIPOL_10V);	// Setting D/A CH2 to range 0V..10V
	IsError();
	DapiDASetMode(handle, 3, DAPI_ADDA_MODE_BIPOL_10V);	// Setting D/A CH3 to range 0V..10V
	IsError();

	DapiADSetMode(handle, 0, DAPI_ADDA_MODE_BIPOL_10V);	// Setting A/D CH0 to range 0V..10V
	IsError();
	DapiADSetMode(handle, 1, DAPI_ADDA_MODE_BIPOL_10V);	// Setting A/D CH1 to range 0V..10V
	IsError();
	DapiADSetMode(handle, 2, DAPI_ADDA_MODE_BIPOL_10V);	// Setting A/D CH2 to range 0V..10V
	IsError();
	DapiADSetMode(handle, 3, DAPI_ADDA_MODE_BIPOL_10V);	// Setting A/D CH3 to range 0V..10V
	IsError();
	
	// --------------------------------
	// Read A/D inputs
	ad_da_data = DapiADGetVolt(handle, 0);	// Reading A/D CH0
	IsError();
	sprintf(msg, "Reading analog input 0 * data = %f\n", ad_da_data);
	log_write(msg, 1);
	
	ad_da_data = DapiADGetVolt(handle, 1);	// Reading A/D CH1
	IsError();
	sprintf(msg, "Reading analog input 1 * data = %f\n", ad_da_data);
	log_write(msg, 1);
	
	ad_da_data = DapiADGetVolt(handle, 2);	// Reading A/D CH2
	IsError();
	sprintf(msg, "Reading analog input 2 * data = %f\n", ad_da_data);
	log_write(msg, 1);
	
	// --------------------------------
	// Set D/A outputs
	ad_da_data = 5.00;
	DapiDASetVolt(handle, 0, ad_da_data);	// Setting D/A CH0 to 5V
	IsError();
	sprintf(msg, "Setting analog output 0 to %fV\n", ad_da_data);
	log_write(msg, 1);
	
	ad_da_data = 7.50;
	DapiDASetVolt(handle, 0, ad_da_data);	// Setting D/A CH0 to 7,5V
	IsError();
	sprintf(msg, "Setting analog output 1 to %fV\n", ad_da_data);
	log_write(msg, 1);

	// --------------------------------
	// Inifinite loop: Reading digital input CH0..7 and write this data to digital output CH0..7
	sprintf(msg, "Starting infinite loop for read/write digital I/Os\n", ad_da_data);
	log_write(msg, 1);
	
	do 
	{
		// --------------------------------
		// Read digital inputs
		data = DapiDIGet8(handle, 0);		// Reading DI CH0..7
		IsError();
		sprintf(msg, "Reading digital input 0-7 * data = 0x%x\n", data);
		log_write(msg, 1);
	
		// --------------------------------
		// Set digital outputs
		DapiDOSet8(handle, 0, data);		// Setting DO CH0..7
		IsError();
		sprintf(msg, "Writing this data to digital output 0-7\n");
		log_write(msg, 1);

		usleep(1*1000*1000);				// 1 sec delay
		
	} while(signal_handler_flag_shutdown != TRUE);
	
	// --------------------------------
	// Shutdown
	
	DapiCloseModule(handle);
	log_write("EXIT -> bye ....", 1);
	
} // main
